"""
Copyright 2023 Marco Ceppi

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from enum import Enum
from pathlib import Path
from typing import Optional

import yaml
from pydantic import BaseModel


class ActionConfig(BaseModel):
    pre: Optional[list[dict[str, str | dict]]]
    post: Optional[list[dict[str, str | dict]]]


class ScreenConfig(BaseModel):
    source: str
    values: Optional[dict]


class YaftiRunModes(str, Enum):
    changed = "run-on-change"
    ignore = "run-once"
    disable = "disabled"


class YaftiProperties(BaseModel):
    path: Optional[Path] = Path("~/.config/yafti/last-run")
    mode: YaftiRunModes = YaftiRunModes.changed


class Config(BaseModel):
    title: str
    properties: YaftiProperties = YaftiProperties()
    actions: Optional[ActionConfig]
    screens: Optional[dict[str, ScreenConfig]]  # Screens are parsed per plugin


def parse(config_file: str) -> Config:
    """Parse the YAML or JSON file passed and return a rendered Config object"""
    with open(config_file) as f:
        cfg = yaml.safe_load(f)
    return Config.parse_obj(cfg)



import asyncio
import hashlib
from functools import partial
from typing import List, Optional

from gi.repository import Adw, Gtk

from yafti import events
from yafti.abc import YaftiScreen, YaftiScreenConfig
from yafti.registry import PLUGINS

_xml = """\
<?xml version="1.0" encoding="UTF-8"?>
<interface>
    <requires lib="gtk" version="4.0"/>
    <requires lib="libadwaita" version="1.0" />
    <template class="YaftiTitleScreen" parent="AdwBin">
        <property name="halign">fill</property>
        <property name="valign">fill</property>
        <property name="hexpand">true</property>
        <child>
            <object class="AdwStatusPage" id="status_page">
                <property name="icon-name">folder</property>
                <property name="title" translatable="yes">Welcome!</property>
                <property name="description" translatable="yes">
                    Make your choices, this wizard will take care of everything.
                </property>
            </object>
        </child>
    </template>
</interface>
"""


@Gtk.Template(string=_xml)
class TitleScreen(YaftiScreen, Adw.Bin):
    __gtype_name__ = "YaftiTitleScreen"

    status_page = Gtk.Template.Child()

    class Config(YaftiScreenConfig):
        title: str
        description: str
        icon: Optional[str] = None
        links: List[dict[str, dict]] = None

    def __init__(
        self,
        title: str = None,
        description: str = None,
        icon: str = None,
        links: List[dict[str, str]] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.status_page.set_title(title)
        self.status_page.set_description(description)

        if links:
            links_list_box = self.render_links_list_box()
            self.append_action_rows(links, links_list_box)

    def render_links_list_box(self):
        links_list_box = Gtk.ListBox()
        links_list_box.set_selection_mode(Gtk.SelectionMode.NONE)
        links_list_box.add_css_class("boxed-list")
        self.status_page.set_child(links_list_box)
        return links_list_box

    def append_action_rows(self, links, links_list_box):
        for link in links:
            title, action = list(link.items())[0]
            plugin, config = list(action.items())[0]
            hash_title = hashlib.md5(
                title.encode("utf-8"), usedforsecurity=False
            ).hexdigest()
            event_name = f"on_action_row_open_{hash_title}"
            event_fn = partial(
                TitleScreen.on_action_row_open, plugin=plugin, config=config
            )

            events.register(event_name)
            events.on(event_name, event_fn)

            def do_emit(*args, **kwargs):
                asyncio.create_task(events.emit(*args, **kwargs))

            _on_clicked = partial(do_emit, event_name)

            link_action_row = Adw.ActionRow()

            action_btn = Gtk.Button()
            action_btn.set_label("Open")
            action_btn.set_valign(Gtk.Align.CENTER)
            action_btn.connect("clicked", _on_clicked)

            link_action_row.set_title(title)
            link_action_row.add_suffix(action_btn)

            links_list_box.append(link_action_row)

    @staticmethod
    async def on_action_row_open(*args, plugin=None, config=None):
        if not plugin and not config:
            return
        await PLUGINS.get(plugin)(config)



from pydantic import BaseModel


class PackageConfig(BaseModel):
    __root__: dict[str, str | dict]


class PackageGroupConfigDetails(BaseModel):
    description: str
    default: bool = True
    packages: list[PackageConfig]


class PackageGroupConfig(BaseModel):
    __root__: dict[str, PackageGroupConfigDetails]



import asyncio
import json

from gi.repository import Gtk
from typing import Optional

import yafti.share
from yafti import events
from yafti import log
from yafti.abc import YaftiScreen
from yafti.screen.console import ConsoleScreen
from yafti.screen.package.state import STATE

_xml = """\
<?xml version="1.0" encoding="UTF-8"?>
<interface>
  <requires lib="gtk" version="4.0"/>
  <template class="YaftiPackageInstallScreen" parent="GtkBox">
    <property name="orientation">vertical</property>
    <child>
      <object class="AdwStatusPage" id="status_page">
        <property name="halign">fill</property>
        <property name="valign">fill</property>
        <property name="hexpand">true</property>
        <property name="title" translatable="yes">Package Installation</property>
      </object>
    </child>
    <child>
        <object class="GtkBox">
            <property name="orientation">vertical</property>
            <property name="halign">fill</property>
            <property name="hexpand">True</property>
            <property name="valign">fill</property>
            <child>
                <object class="GtkProgressBar" id="pkg_progress">
                <property name="margin-start">40</property>
                <property name="margin-end">40</property>
                </object>
            </child>
            <child>
                <object class="GtkBox">
                    <property name="orientation">horizontal</property>
                    <child>
                        <object class="GtkBox">
                            <property name="halign">fill</property>
                            <property name="hexpand">true</property>
                        </object>
                    </child>
                    <child>
                        <object class="GtkButton" id="btn_console">
                            <property name="visible">True</property>
                            <property name="margin-end">40</property>
                            <property name="margin-top">20</property>
                            <property name="margin-bottom">20</property>
                            <property name="label">Show Console</property>
                        </object>
                    </child>
                </object>
            </child>
        </object>
    </child>
  </template>
</interface>
"""


@Gtk.Template(string=_xml)
class PackageInstallScreen(YaftiScreen, Gtk.Box):
    __gtype_name__ = "YaftiPackageInstallScreen"

    status_page = Gtk.Template.Child()
    pkg_progress = Gtk.Template.Child()
    btn_console = Gtk.Template.Child()
    started = False
    already_run = False
    pulse = True

    def __init__(
        self,
        title: str = "Package Installation",
        package_manager: str = "yafti.plugin.flatpak",
        package_manager_defaults: Optional[dict] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        from yafti.registry import PLUGINS

        self.status_page.set_title(title)
        self.package_manager = PLUGINS.get(package_manager)
        self.package_manager_defaults = package_manager_defaults or {}
        self.btn_console.connect("clicked", self.toggle_console)

    async def on_activate(self):
        if self.started or self.already_run:
            return
        self.console = ConsoleScreen()
        self.started = True
        events.on("btn_next", self.next)
        await self.draw()

    async def next(self, _):
        return self.started

    def toggle_console(self, btn):
        btn.set_label("Show Console" if self.console.get_visible() else "Hide Console")
        self.console.toggle_visible()

    async def do_pulse(self):
        self.pkg_progress.set_pulse_step(1.0)
        while self.pulse:
            self.pkg_progress.pulse()
            await asyncio.sleep(0.5)

    def draw(self):
        self.console.hide()
        self.append(self.console)
        packages = [item.replace("pkg:", "") for item in STATE.get_on("pkg:")]
        asyncio.create_task(self.do_pulse())
        return self.install(packages)

    def run_package_manager(self, packge_config):
        try:
            config = json.loads(packge_config)
        except json.decoder.JSONDecodeError as e:
            log.debug("could not parse", config=packge_config, e=e)
            config = {"package": packge_config}

        log.debug("parsed packages config", config=config)
        opts = self.package_manager_defaults.copy()
        opts.update(config)
        return self.package_manager.install(**opts)

    async def install(self, packages: list):
        total = len(packages)
        yafti.share.BTN_NEXT.set_label("Installing...")
        yafti.share.BTN_BACK.set_visible(False)
        for idx, pkg in enumerate(packages):
            results = await self.run_package_manager(pkg)
            self.console.stdout(results.stdout)
            self.console.stderr(results.stderr)
            self.pulse = False
            self.pkg_progress.set_fraction((idx + 1) / total)

        self.console.stdout(b"Installation Complete!")

        self.started = False
        self.already_run = True
        yafti.share.BTN_NEXT.set_label("Next")
        yafti.share.BTN_BACK.set_visible(True)



"""
Copyright 2023 Marco Ceppi

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

\f

Install, remove, list, and manage flatpaks

Configuration usage example:

  commands:
    pre:
    # Install a Flatpak package
    - yafti.plugin.flatpak: com.github.marcoceppi.PackageName
    # Explicit install
    - yafti.plugin.flatpak:
        install: com.github.marcoceppi.PackageName
    # Install with options
    - yafti.plugin.flatpak:
        install:
          pkg: com.github.marcoceppi.PackageName
          update: true
          user: true
    # Remove a flatpak package
    - yafti.plugin.flatpak:
        remove: com.github.marcoceppi.PackageName
    # Remove with options
    - yafti.plugin.flatpak:
        remove:
          pkg: com.github.marcoceppi.PackageName
          force: true

Programmatic usage example:

  from yafti.plugin.flatpak import Flatpak
  f = Flatpak()
  f.install("com.github.marcoceppi.PackageName")
  f.install(pkg="com.github.marcoceppi.PackageName", reinstall=True)

  f("com.github.marcoceppi.PackageName")
  f(install="com.github.marcoceppi.PackageName")
  f(install={"pkg": "com.github.marcoceppi.PackageName", "reinstall": True})

  f.remove("com.github.marcoceppi.PackageName")
  f.remove(pkg="com.github.marcoceppi.PackageName", force=True)

  f(remove="com.github.marcoceppi.PackageName")
  f(remove={"pkg": "com.github.marcoceppi.PackageName", "force": True})
"""

import asyncio
from typing import Any, Optional

from pydantic import BaseModel, ValidationError, root_validator

from yafti.abc import YaftiPluginReturn
from yafti.plugin.run import Run


class ApplicationDetail(BaseModel):
    """Flatpak application information"""

    id: str
    name: str
    version: str
    branch: str
    installation: str


class FlatpakException(Exception):
    """Flatpak binary encountered a problem"""


class FlatpakInstallError(FlatpakException):
    """Flatpak package install failed"""


class FlatpakRemoveError(FlatpakException):
    """Flatpak package removal failed"""


class Flatpak(Run):
    """
    Install, remove, list, and manage flatpaks

    Attributes:
        bin: The full POSIX path to the flatpak binary on disk
    """

    class Scheme(BaseModel):
        """Flatpak plugin configuration validation"""

        install: Optional[str | dict] = None
        remove: Optional[str | dict] = None

        @root_validator
        def must_have_atleast_one(cls, values):
            """Validate one, and only one, key is passed

            Returns:
                A dict of the already parsed values from Pydantic

            Raises:
                ValueError: A violation of the validation rule.
            """
            if values.get("install") is None and values.get("remove") is None:
                raise ValueError("Either install or remove is required")
            if values.get("install") is not None and values.get("remove") is not None:
                raise ValueError("Only a single install or remove can be passed")
            return values

    def __init__(self):
        """Verify that flatpak binary exists on the host machine"""
        self.bin = "/usr/bin/flatpak"

    def validate(self, options: Any) -> Scheme:
        """Sanitize and validate inputs

        Args:
            options: Plugin arguments to be validated and sanitized

        Returns:
            An object of the parsed input options

        Raises:
            ValidationError: Input could not be sanitized and did not conform to
                             validation rules
        """
        if isinstance(options, str):
            options = {"install": options}

        return self.Scheme.parse_obj(options)

    def _parse_args(self, **kwargs):
        """Map a series of boolean keyword arguments to command-line flags"""
        arg_map = {"update": "or-update"}

        return [f"--{arg_map.get(k, k)}" for k, v in kwargs.items() if v is True]

    async def install(
        self,
        package: str,
        user: bool = True,
        system: bool = False,
        assumeyes: bool = True,
        reinstall: bool = False,
        noninteractive: bool = True,
        update: bool = True,
    ) -> YaftiPluginReturn:
        """Install flatpak package on the host system

        Args:
          package: Name of the flatpak package to install
          user: Install on the user installation
          system: Install on the system-wide installation
          assumeyes:

        Returns:
          An object containing the stdout and stderr from the flatpak command

        Raises:
          FlatpakInstallError: An error occurred trying to install the Flatpak
        """
        args = self._parse_args(
            user=user,
            system=system,
            assumeyes=assumeyes,
            reinstall=reinstall,
            update=update,
            noninteractive=noninteractive,
        )
        cmd = [self.bin, "install"]
        cmd.extend(args)
        cmd.append(package)
        return await self.exec(" ".join(cmd))

    async def remove(
        self,
        package: str,
        user: bool = False,
        system: bool = True,
        force: bool = False,
        noninteractive: bool = True,
    ) -> YaftiPluginReturn:
        """Remove flatpak package on the host system"""
        args = self._parse_args(
            user=user, system=system, force=force, noninteractive=noninteractive
        )
        cmd = [self.bin, "remove"]
        cmd.extend(args)
        cmd.append(package)
        return self.exec(cmd)

    def ls(self) -> list[ApplicationDetail]:
        pass

    def __call__(self, options) -> YaftiPluginReturn:
        try:
            params = self.validate(options)
        except ValidationError as e:
            return YaftiPluginReturn(errors=str(e), code=1)

        # TODO: when a string is passed, make sure it maps to the "pkg" key.
        if params.install:
            if isinstance(params.install, str):
                params.install = {"package": params.install}
            r = asyncio.ensure_future(self.install(**params.install))
        else:
            if isinstance(params.remove, str):
                params.remove = {"package": params.remove}
            r = asyncio.ensure_future(self.remove(**params.install))
        return YaftiPluginReturn(output=r.stdout, errors=r.stderr, code=r.returncode)



"""
Copyright 2023 Marco Ceppi

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging

import typer
import yaml

import yafti.setup  # noqa
from yafti import log
from yafti.app import Yafti
from yafti.parser import Config


def run(config: typer.FileText = typer.Argument("/etc/yafti.yml"), debug: bool = False):
    log.set_level(logging.DEBUG if debug else logging.INFO)
    log.debug("starting up", config=config, debug=debug)
    config = Config.parse_obj(yaml.safe_load(config))
    app = Yafti(config)
    app.run(None)


def app():
    typer.run(run)


if __name__ == "__main__":
    app()



# Copyright 2023 Marco Ceppi
# SPDX-License-Identifier: Apache-2.0

"""
Present the user with confirmation (consent) to proceed with modifications
on their system

Configuration usage example:

  screens:
    can-we-modify-your-flatpaks:
      source: yafti.screen.consent
      values:
        title: Welcome traveler
        condition:
          run: flatpak remotes --system | grep fedora
        description: |
          This tool modifies your flatpaks and flatpak sources.
          If you do not want to do this exit the installer.
          For new users just do it (tm)
        actions:
          - run: flatpak remote-delete fedora --force
          - run: flatpak remove --system --noninteractive --all

Configuration:

* title: Header of the screen
* description: long form text
* condition: dict of plugin: plugin config. Plugin must return a 0 code to display
             screen. Any other code will result in the screen being skipped
* actions: list of plugins to execute once screen is accepted
"""

import asyncio
from typing import Optional

from gi.repository import Adw, Gtk

import yafti.share
from yafti import events
from yafti.abc import YaftiScreen, YaftiScreenConfig
from yafti.registry import PLUGINS

_xml = """\
<?xml version="1.0" encoding="UTF-8"?>
<interface>
    <requires lib="gtk" version="4.0"/>
    <requires lib="libadwaita" version="1.0" />
    <template class="YaftiConsentScreen" parent="AdwBin">
        <property name="halign">fill</property>
        <property name="valign">fill</property>
        <property name="hexpand">true</property>
        <child>
            <object class="AdwStatusPage" id="status_page">
                <property name="title" translatable="yes">Welcome!</property>
                <property name="description" translatable="yes">
                    Make your choices, this wizard will take care of everything.
                </property>
            </object>
        </child>
    </template>
</interface>
"""


@Gtk.Template(string=_xml)
class ConsentScreen(YaftiScreen, Adw.Bin):
    __gtype_name__ = "YaftiConsentScreen"

    status_page = Gtk.Template.Child()

    class Config(YaftiScreenConfig):
        title: str
        description: str
        actions: Optional[list[dict[str, str | dict]]] = None

    def __init__(
        self,
        title: str = None,
        description: str = None,
        actions: list = None,
        condition: dict = None,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.status_page.set_title(title)
        self.status_page.set_description(description)
        self.actions = actions
        self.condition = condition
        self.already_run = False

    async def on_activate(self):
        events.on("btn_next", self.next)
        yafti.share.BTN_NEXT.set_label("Accept")

    async def on_deactivate(self):
        events.detach("btn_next", self.next)

    async def next(self, _):
        if self.already_run:
            return False

        to_run = []
        for action in self.actions:
            plugin_name = list(action.keys())[0]
            plugin = PLUGINS.get(plugin_name)
            to_run.append(plugin(action[plugin_name]))
        await asyncio.gather(*to_run)
        self.already_run = True
        return False



from typing import Optional

from gi.repository import Adw, Gtk

import yafti.share
from yafti import events
from yafti.abc import YaftiScreen, YaftiScreenConfig
from yafti.screen.package.models import PackageConfig, PackageGroupConfig
from yafti.screen.package.screen import PackageInstallScreen, PackagePickerScreen
from yafti.screen.package.state import STATE
from yafti.screen.package.utils import parse_packages

_xml = """\
<?xml version="1.0" encoding="UTF-8"?>
<interface>
  <requires lib="gtk" version="4.0"/>
  <template class="YaftiPackageScreen" parent="AdwBin">
    <property name="halign">fill</property>
    <property name="valign">fill</property>
    <property name="hexpand">true</property>
    <child>
      <object class="AdwCarousel" id="pkg_carousel">
        <property name="vexpand">True</property>
        <property name="hexpand">True</property>
        <property name="allow_scroll_wheel">False</property>
        <property name="allow_mouse_drag">False</property>
        <property name="allow_long_swipes">False</property>
      </object>
    </child>
  </template>
</interface>
"""


@Gtk.Template(string=_xml)
class PackageScreen(YaftiScreen, Adw.Bin):
    __gtype_name__ = "YaftiPackageScreen"

    pkg_carousel = Gtk.Template.Child()

    class Config(YaftiScreenConfig):
        title: str
        show_terminal: bool = True
        package_manager: str
        groups: Optional[PackageGroupConfig] = None
        packages: Optional[list[PackageConfig]] = None
        package_manager_defaults: Optional[dict] = None

    def __init__(
        self,
        title: str = "Package Installation",
        package_manager: str = "yafti.plugin.flatpak",
        packages: list[PackageConfig] = None,
        groups: PackageGroupConfig = None,
        show_terminal: bool = True,
        package_manager_defaults: Optional[dict] = None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.title = title
        self.packages = groups or packages
        self.show_terminal = show_terminal
        self.package_manager = package_manager
        self.package_manager_defaults = package_manager_defaults
        STATE.load(parse_packages(self.packages))
        self.pkg_carousel.connect("page-changed", self.changed)
        self.draw()

    def draw(self):
        self.pkg_carousel.append(
            PackagePickerScreen(title=self.title, packages=self.packages)
        )
        self.pkg_carousel.append(
            PackageInstallScreen(
                title=self.title,
                package_manager=self.package_manager,
                package_manager_defaults=self.package_manager_defaults,
            )
        )

    def on_activate(self):
        events.on("btn_next", self.next)
        events.on("btn_back", self.back)
        yafti.share.BTN_NEXT.set_label("Install")

    def on_deactivate(self):
        events.detach("btn_next", self.next)
        events.detach("btn_back", self.back)

    @property
    def idx(self):
        return self.pkg_carousel.get_position()

    @property
    def total(self):
        return self.pkg_carousel.get_n_pages()

    def goto(self, page: int, animate: bool = True):
        if page < 0:
            page = 0

        if page >= self.pkg_carousel.get_n_pages():
            page = self.pkg_carousel.get_n_pages()

        current_screen = self.pkg_carousel.get_nth_page(self.idx)
        next_screen = self.pkg_carousel.get_nth_page(page)

        current_screen.deactivate()
        self.pkg_carousel.scroll_to(next_screen, animate)

    def changed(self, *args):
        current_screen = self.pkg_carousel.get_nth_page(self.idx)
        current_screen.activate()

    async def next(self, _):
        if not self.active:
            return False
        if self.idx + 1 == self.total:
            return False
        self.goto(self.idx + 1)

    async def back(self, _):
        if not self.active:
            return False

        if self.idx - 1 < 0:
            return False
        self.goto(self.idx - 1)



# Copyright 2023 Marco Ceppi
# SPDX-License-Identifier: Apache-2.0

from importlib.metadata import entry_points

_plugins = entry_points(group="yafti.plugin")
PLUGINS = {s.name: s.load()() for s in _plugins}

_screens = entry_points(group="yafti.screen")
SCREENS = {s.name: s.load() for s in _screens}



"""
Copyright 2023 Marco Ceppi

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import asyncio
from inspect import iscoroutinefunction
from typing import Any, Optional

from pydantic import BaseModel


class YaftiPlugin:
    pass


async def _show_screen(condition):
    from yafti.registry import PLUGINS

    plugin_name = list(condition.keys())[0]
    plugin = PLUGINS.get(plugin_name)
    result = await plugin(condition[plugin_name])
    return result.code == 0


class YaftiScreenConfig(BaseModel):
    condition: Optional[dict[str, str | dict]] = None


class YaftiScreen:
    active = False

    class Config(YaftiScreenConfig):
        pass

    @classmethod
    async def from_config(cls, cfg: Any):
        c = cls.Config.parse_obj(cfg)
        show = True
        if c.condition:
            show = await _show_screen(c.condition)
        if not show:
            return None

        return cls(**c.dict(exclude={"condition"}))

    def activate(self):
        self.active = True
        if hasattr(self, "on_activate"):
            if iscoroutinefunction(self.on_activate):
                asyncio.ensure_future(self.on_activate())
            else:
                self.on_activate()

    def deactivate(self):
        self.active = False
        if hasattr(self, "on_deactivate"):
            if iscoroutinefunction(self.on_deactivate):
                asyncio.ensure_future(self.on_deactivate())
            else:
                self.on_deactivate()


class YaftiPluginReturn(BaseModel):
    output: Optional[str | list[str]]
    errors: Optional[str | list[str]]
    code: int = 0



"""
Copyright 2023 Marco Ceppi

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import hashlib

import gbulb
import yaml
from gi.repository import Adw
from pathlib import Path

from yafti.parser import Config, YaftiRunModes
from yafti.screen.window import Window


class Yafti(Adw.Application):
    def __init__(self, cfg: Config = None, loop=None):
        super().__init__(application_id="it.ublue.Yafti")
        self.config = cfg
        self.loop = loop or gbulb.get_event_loop()

    def run(self, *args, **kwargs):
        configured_mode = self.config.properties.mode
        _p: Path = self.config.properties.path.expanduser()
        # TODO(GH-#103): Remove this prior to 1.0 release. Start.
        _old_p = Path("~/.config/yafti-last-run").expanduser()
        if _old_p.exists() and _old_p.resolve() != _p.resolve():
            if not _p.parent.is_dir():
                _p.parent.mkdir(mode=0o755, parents=True, exist_ok=True)
            if _p.is_file():
                _p.unlink()
            _old_p.rename(_p)
        # TODO(GH-#103): End.
        if configured_mode == YaftiRunModes.disable:
            return

        if configured_mode == YaftiRunModes.changed:
            if _p.exists() and _p.read_text() == self.config_sha:
                return

        if configured_mode == YaftiRunModes.ignore and _p.exists():
            return

        super().run(*args, **kwargs)

    def do_activate(self):
        win = Window(application=self)
        win.present()
        self.loop.run()

    @property
    def config_sha(self):
        return hashlib.sha256(yaml.dump(self.config.dict()).encode()).hexdigest()

    def sync_last_run(self):
        p = self.config.properties.path.expanduser()
        if not p.parent.is_dir():
            p.parent.mkdir(mode=0o755, parents=True, exist_ok=True)
        p.write_text(self.config_sha)

    def quit(self, *args, **kwargs):
        self.loop.stop()
        self.sync_last_run()
        super().quit()
